class Revelry::Content::ContentsController < ApplicationController
  respond_to :html, :json
  layout 'layouts/revelry_content/application'
  include Revelry::Content::WithRevelryContent
  include Revelry::Content::ControllerAuthorization

  before_action :check_authorization, except: :index

  after_action :export_javascript, only: :create, if: proc { Revelry::Content.config.js_export? }

  def index
    respond_with data: Revelry::Content::Content.all.as_json
  end

  def create
    @content = Revelry::Content::Content.find_or_initialize_by(key: key_param)
    @content.last_whodunnit = Revelry::Content.config.user_for_content.call(self).to_s
    @content.update(content_params)

    respond_with @content do |fmt|
      fmt.html { redirect_to :back }
      fmt.json { render json: @content }
    end
  end

  def import
    # Parse file into a hash
    content_objs = JSON.parse(import_file_param.read)
    content_objs.each do |key, content_object|
      Revelry::Content::Content.from_import_hash(content_object)
    end
    redirect_to :back
  end

  def export
    # i.e. content-2015-01-23-142345.json
    filename = DateTime.now.strftime('content-%Y-%m-%d-%H%M%S.json')
    respond_with Revelry::Content::Content.all do |fmt|
      fmt.json do
        # send the file as a file download (instead of rendering) with a json
        # mime type
        send_data Revelry::Content::Content.as_serialized_table.to_json, filename: filename
      end
    end
  end

  protected

  def check_authorization
    throw "Unauthorized" unless Revelry::Content.config.authorize(self)
  end

  def content_params
    params.require(:content).permit(:content, :src)
  end

  def key_param
    params.require(:content).permit(:key).try(:[],:key)
  end

  def import_file_param
    params.require(:import)
  end

  def export_javascript
    Revelry::Content::JsExporter.write_export
  end
end
